from damster.utils import initialize_logger, get_config
from damster.reports.bamboo import BambooDeploymentsReport, BambooBuildsReport, BambooDBDeploymentPermissions
from damster.reports.confluence import ConfluenceChanges
import click

log = initialize_logger(__name__)


@click.group()
@click.version_option()
# @click.option('--debug/--no-debug', default=False)
@click.option('--config', '-c', required=False, help='configuration file to use')
@click.pass_context
def cli(ctx, config):
    """Damster: Reports from Atlassian tools.
    """
    ctx.obj = get_config(config)


@cli.group()
def bamboo():
    """Bamboo reports."""
    pass


@bamboo.command('deployments', short_help='generate deployments report')
@click.argument('from-date')
@click.argument('to-date', required=False)
@click.option('--use-cache/--no-use-cache', default=False)
@click.pass_context
def bamboo_deployments(ctx, from_date, to_date, use_cache):
    """Generate a deployments report"""

    click.echo('Getting Bamboo deployments between {} and {}'.format(from_date, to_date))
    report = BambooDeploymentsReport(
        ctx.obj,
        from_date=from_date,
        to_date=to_date
    )
    report.run_report(use_cache=use_cache)


@bamboo.command('builds', short_help='generate build report')
@click.argument('from-date')
@click.argument('to-date', required=False)
@click.option('--use-cache/--no-use-cache', default=False)
@click.pass_context
def bamboo_builds(ctx, from_date, to_date, use_cache):
    """Generate a builds report"""

    click.echo('Getting Bamboo builds between {} and {}'.format(from_date, to_date))
    report = BambooBuildsReport(
        ctx.obj,
        from_date=from_date,
        to_date=to_date
    )
    report.run_report(use_cache=use_cache)


@bamboo.command('deployment-permissions', short_help='generate deployment permissions report')
@click.option('--use-ssh-tunnel/--no-use-ssh-tunnel', '-S', default=False)
@click.pass_context
def bamboo_deployment_permissions(ctx, use_ssh_tunnel):
    """Deployment projects and environments permissions"""

    bamboo_report = BambooDBDeploymentPermissions(ctx.obj, use_ssh_tunnel=use_ssh_tunnel)
    bamboo_report.save_to_csv()
    bamboo_report.save_to_json()
    bamboo_report.save_to_html(title='Bamboo Reports: Deployment Permissions')


@cli.group()
def confluence():
    """Confluence reports."""
    pass


@confluence.command('changes', short_help='generate changes report')
@click.argument('from-date', required=False)
@click.argument('to-date', required=False)
@click.option('--use-ssh-tunnel/--no-use-ssh-tunnel', '-S', default=False)
@click.pass_context
def confluence_changes(ctx, from_date, to_date, use_ssh_tunnel):
    """Confluence content changes"""

    confluence_report = ConfluenceChanges(ctx.obj, from_date, to_date, use_ssh_tunnel=use_ssh_tunnel)
    confluence_report.save_to_csv()
    confluence_report.save_to_json()
    # confluence_report.save_to_html(title='Confluence Reports: Changes')



#!/bin/env python
from damster.utils import initialize_logger, get_config
from damster.reports.bamboo import BambooDBDeploymentPermissions

log = initialize_logger(__name__)


cfg = get_config()


def main():

    bamboo_report = BambooDBDeploymentPermissions(cfg, use_ssh_tunnel=False)
    bamboo_report.save_to_csv()
    bamboo_report.save_to_json()
    bamboo_report.save_to_html(title='Bamboo Reports: Deployment Permissions')


if __name__ == "__main__":
    main()



from damster.utils import initialize_logger, quoted, time_to_excel, replace_non_ascii
from damster.reports.bamboo.utils import TriggerReason
from damster.reports.base_report import BaseReport
from atlassian import Bamboo
import arrow
import jinja2
from distutils.dir_util import mkpath
from six.moves.urllib.parse import unquote


log = initialize_logger(__name__)


def prettify_date_time(dt):
    if len(dt) == 24:
        return dt[:-5].replace('T', ' ')
    return dt


class BambooBuildsReport(BaseReport):

    def __init__(self, cfg, from_date, to_date=None, name='bamboo_builds_report'):

        super(BambooBuildsReport, self).__init__(cfg, from_date, to_date, name)
        self.bamboo = Bamboo(**cfg['Bamboo'])

    # TODO: Needs refactoring
    def generate_report(self):
        if self._report is None:
            log.info('Starting report generation for build results between {} and {}'.format(
                self._time_to_string(self.from_date), self._time_to_string(self.to_date)
            ))
            report = list()
            for project in self.bamboo.projects():
                log.info('Retrieving plans for project "{}:{}"'.format(project['key'], project['name']))
                project_description = project['description'] if 'description' in project else ''
                project_dict = dict(
                    key=project['key'],
                    name=project['name'],
                    description=project_description,
                    plans=list()
                )
                for plan in self.bamboo.project_plans(project_key=project['key']):
                    log.info('Retrieving branches for plan "{}:{}"'.format(plan['key'], plan['shortName']))
                    plan_dict = dict(
                        short_name=plan['shortName'],
                        short_key=plan['shortKey'],
                        enabled=plan['enabled'],
                        type=plan['type'],
                        key=plan['key'],
                        name=plan['name'],
                        branches=list()
                    )
                    log.debug(plan_dict)
                    for branch in self.bamboo.search_branches(plan['key']):
                        log.info('Retrieving results for branch "{}"'.format(branch['searchEntity']['branchName']))
                        branch_dict = dict(
                            key=branch['id'],
                            type=branch['type'],
                            name=branch['searchEntity']['branchName'],
                            results=list()
                        )
                        for result in self.bamboo.results(project_key=branch['id'], expand='results'):
                            log.debug('Build result: {}'.format(result))
                            result_dict = dict(
                                number=result['number'],
                                state=result['state'],
                                key=result['key'],
                                lifecycle_state=result['lifeCycleState'],
                            )
                            result_details = self.bamboo.build_result(build_key=result['buildResultKey'])
                            # WTF Atlassian! buildStartedTime is not always stored
                            started_time_string = result_details.get('buildStartedTime',
                                                                     result_details.get('buildCompletedTime'))
                            started_time = arrow.get(started_time_string).replace(tzinfo=self.time_zone)
                            if started_time > self.from_date:
                                if started_time < self.to_date:
                                    log.debug('Result details: {}'.format(result_details))
                                    result_dict['started'] = prettify_date_time(started_time_string)
                                    result_dict['finished'] = prettify_date_time(
                                        result_details.get('buildCompletedTime', ''))
                                    result_dict['duration'] = result_details.get('buildDuration', 0)
                                    trigger, user_name, user_id, build_id = TriggerReason(
                                        result_details['reasonSummary']).tuple
                                    result_dict['trigger_raw'] = unquote(result_details['reasonSummary'])
                                    result_dict['trigger_type'] = trigger
                                    result_dict['trigger_user'] = user_name
                                    result_dict['trigger_user_id'] = user_id
                                    result_dict['trigger_build'] = build_id
                                    result_dict['number_stages'] = result_details['stages']['size']
                                    result_dict['number_issues'] = result_details['jiraIssues']['size']
                                    result_dict['number_artifacts'] = result_details['artifacts']['size']
                                    branch_dict['results'].append(result_dict)
                            else:
                                break
                            branch_dict['successful'] = len(
                                [r for r in branch_dict['results'] if r['state'] == 'Successful'])
                            branch_dict['failed'] = len(
                                [r for r in branch_dict['results'] if r['state'] == 'Failed'])
                            branch_dict['unknown'] = (len(branch_dict['results']) -
                                                      branch_dict['failed'] - branch_dict['successful'])
                        if branch_dict['results']:
                            plan_dict['branches'].append(branch_dict)
                    if plan_dict['branches']:
                        plan_dict['successful'] = sum([b['successful'] for b in plan_dict['branches']])
                        plan_dict['failed'] = sum([b['failed'] for b in plan_dict['branches']])
                        plan_dict['unknown'] = sum([b['unknown'] for b in plan_dict['branches']])
                        project_dict['plans'].append(plan_dict)
                if project_dict['plans']:
                    project_dict['successful'] = sum([b['successful'] for b in project_dict['plans']])
                    project_dict['failed'] = sum([b['failed'] for b in project_dict['plans']])
                    project_dict['unknown'] = sum([b['unknown'] for b in project_dict['plans']])
                    report.append(project_dict)
            self._report = report
        return self._report

    def save_to_html(self, template_name=None, **args):
        out_html = self.output_file(ext='html')
        log.info('Saving to HTML: {}'.format(out_html))
        template_name = template_name or self.name + '.html'
        jinja_env = jinja2.Environment(loader=jinja2.PackageLoader('damster', 'templates'))
        template = jinja_env.get_template(template_name)

        summary = dict(
            projects=len(self.report),
            successful=sum([pr['successful'] for pr in self.report]),
            failed=sum([pr['failed'] for pr in self.report]),
            unknown=sum([pr['unknown'] for pr in self.report]),
            from_date=self._time_to_string(self.from_date, fmt='YYYY/MM/DD'),
            to_date=self._time_to_string(self.to_date, fmt='YYYY/MM/DD'),
            bamboo_url=self.bamboo.url
        )

        html = template.render(
            report=self.report,
            summary=summary,
            **args
        )
        mkpath(self.output_folder)
        with open(out_html, 'w', encoding='utf8') as outfile:
            outfile.write(html)

    def save_to_csv(self):
        out_csv = self.output_file(ext='csv')
        log.info('Saving to CSV file {}'.format(out_csv))
        lines = list()
        header = ','.join([
            'project_key',
            'project_name',
            'plan_key',
            'plan_name',
            'branch_name',
            'branch_type',
            'build_key',
            'status',
            'started',
            'finished',
            'duration',
            'trigger',
            'trigger_type',
            'trigger_user',
            'trigger_user_id',
            'trigger_build',
            'trigger_raw',
            'stages',
            'artifacts',
            'jira_issues'
        ])
        lines.append(header)
        for project in self.report:
            for plan in project['plans']:
                for branch in plan['branches']:
                    for result in branch['results']:
                        try:
                            line = ','.join([
                                project['key'],
                                quoted(project['name']),
                                plan['short_key'],
                                quoted(plan['name']),
                                quoted(branch['name']),
                                branch['type'],
                                result['key'],
                                result['state'],
                                time_to_excel(result['started']),
                                time_to_excel(result['finished']),
                                str(result['duration']),
                                result['trigger_type'],
                                result['trigger_type'],
                                result['trigger_user'],
                                result['trigger_user_id'],
                                result['trigger_build'],
                                result['trigger_raw'].replace('\n', ' - '),
                                str(result['number_stages']),
                                str(result['number_artifacts']),
                                str(result['number_issues'])
                            ])
                            lines.append(line)
                        except TypeError as e:
                            log.error(e)
                            log.error(result)
        mkpath(self.output_folder)
        with open(out_csv, 'w', encoding='ascii') as outfile:
            outfile.write(replace_non_ascii('\n'.join(lines), '_'))

    def run_report(self, use_cache=True):
        super(BambooBuildsReport, self).run_report(use_cache=use_cache)
        self.save_to_csv()
        self.save_to_html(title='Bamboo Reports: Builds')



from damster.utils import initialize_logger, time_to_excel, quoted
from damster.reports.db_query import GenericDB
from atlassian import Crowd
from functools import lru_cache
from distutils.dir_util import mkpath

log = initialize_logger(__name__)


class ConfluenceChanges(GenericDB):
    query_changes = """
        SELECT c.contentid,
               c.contenttype,
               c.title,
               c.version,
               c.creationdate,
               c.lastmoddate,
               c.content_status,
               c.spaceid,
               c.prevver,
               um.username,
               sp.spacekey,
               sp.spacename
        FROM   PUBLIC.content c
        JOIN   PUBLIC.user_mapping um
        ON     c.lastmodifier = um.user_key
        LEFT JOIN   PUBLIC.spaces sp
        ON     c.spaceid = sp.spaceid
        WHERE  c.contenttype ='PAGE'
        /*AND    c.content_status = 'current'*/
        {time_constraint}
        ORDER BY sp.spacename, c.title, c.lastmoddate
        """

    name = 'confluence_changes'

    db_fields = [
        'c_contentid',
        'c_contenttype',
        'c_title',
        'c_version',
        'c_creationdate',
        'c_lastmoddate',
        'c_content_status',
        'c_spaceid',
        'c_prevver',
        'um_username',
        'sp_spacekey',
        'sp_spacename'
        ]
    additional_fields = [
        'version_diff',
        'excel_created',
        'excel_modified',
        'user_name'
    ]

    def __init__(self, cfg, from_date, to_date,
                 db_settings_section='Confluence DB',
                 name='confluence_changes',
                 use_ssh_tunnel=False):
        super(ConfluenceChanges, self).__init__(
            cfg, db_settings_section, name, use_ssh_tunnel=use_ssh_tunnel)
        self.from_date = from_date
        self.to_date = to_date
        self.crowd = Crowd(**cfg['Crowd'])

    @lru_cache(maxsize=50)
    def _get_space_info_for_draft(self, prevver):
        query = """
        SELECT sp.spacekey,
               sp.spacename,
               sp.spaceid
        FROM   PUBLIC.spaces sp
               JOIN PUBLIC.content c
                 ON c.spaceid = sp.spaceid
        WHERE  c.contentid = {prevver}
        """.format(prevver=prevver)
        if prevver:
            return self.exec_query(query=query)[0]
        return None, None, None

    def _get_display_name(self, user_id):
        try:
            user_details = self.crowd.user(user_id)
            return user_details['display-name']
        except Exception:
            return user_id

    def __version_change_diff(self, content):
        base_url = self.cfg['Confluence'].get('url', 'http://localhost:8090')
        if content['c_version'] == '1':
            url = '{base_url}/pages/?pageId={page_id}'
        else:
            url = "{base_url}/pages/diffpagesbyversion.action?" \
                  "pageId={page_id}&" \
                  "selectedPageVersions={version_number_1}&selectedPageVersions={version_number_2}"
        return url.format(
            base_url=base_url,
            page_id=content['c_contentid'],
            version_number_1=content['c_version'],
            version_number_2=str(int(content['c_version'])-1)
        )

    def generate_report(self):
        time_constraint = "AND c.lastmoddate > CURRENT_DATE - interval '1 months'"
        if self.from_date:
            time_constraint = " AND c.lastmoddate > {}".format(self.from_date)
        if self.to_date:
            time_constraint = time_constraint + " AND c.lastmoddate > {}".format(self.to_date)

        query = self.query_changes.format(time_constraint=time_constraint)
        confluence_changes = self.exec_query(query=query)

        report = list()
        for row in confluence_changes:
            zp = list(zip(self.db_fields, row))
            report_row = {k: str(v) for k, v in zp}
            report_row['version_diff'] = self.__version_change_diff(report_row)
            report_row['excel_created'] = time_to_excel(report_row['c_creationdate'])
            report_row['excel_modified'] = time_to_excel(report_row['c_lastmoddate'])
            report_row['user_name'] = self._get_display_name(report_row['um_username'])
            log.debug(report_row)
            if report_row['sp_spacekey'] == 'None':
                space_key, space_name, space_id = self._get_space_info_for_draft(report_row['c_prevver'])
                report_row['sp_spacekey'] = space_key
                report_row['sp_spacename'] = space_name
                report_row['c_spaceid'] = space_id
            report.append(report_row)
        return report

    def save_to_csv(self, output_file=None):
        out_csv = output_file or self.output_file(ext='csv')
        log.info('Saving to CSV file {}'.format(out_csv))
        lines = [','.join(self.db_fields + self.additional_fields)]
        for line in self.report:
            lines.append(','.join([quoted(f) for f in line.values()]))

        mkpath(self.output_folder)
        with open(out_csv, 'w', encoding='utf8') as outfile:
            outfile.write('\n'.join(lines))



from damster.utils import initialize_logger
from sshtunnel import SSHTunnelForwarder
from getpass import getuser
import psycopg2
import jinja2
import os
import json
from distutils.dir_util import mkpath
import sys

log = initialize_logger(__name__)


class GenericDB(object):
    """
    Generic DB query object
    """
    table_columns = ()

    query = ''

    def __init__(self, cfg, db_settings_section='Common', name='generic_db_query', use_ssh_tunnel=False):
        self.name = name
        self.cfg = cfg
        self.ssh_tunnel = None
        self.db_settings = self.cfg[db_settings_section]

        if use_ssh_tunnel:
            self.ssh_tunnel = self.start_ssh_tunnel()
        self.db = self.connect()
        self.cur = self.db.cursor()
        self._report = None

    def start_ssh_tunnel(self):
        ssh_settings = self.cfg['SSH']
        host = self.db_settings['ssh_gateway']
        port = ssh_settings.get('port', 22)
        remote_bind_address = (self.db_settings.get('host'), int(self.db_settings.get('port', 5432)))
        local_bind_address = ('localhost', int(ssh_settings.get('local_bind_port', 6543)))
        tunnel = SSHTunnelForwarder(
            (host, port),
            ssh_username=ssh_settings.get('ssh_username', getuser()),
            ssh_private_key=os.path.expanduser('~/.ssh/{}'.format(
                ssh_settings.get('ssh_private_key', 'id_rsa'))),
            remote_bind_address=remote_bind_address,
            local_bind_address=local_bind_address
        )
        try:
            tunnel.start()
            return tunnel
        except Exception as e:
            log.error('Error starting SSH tunnel to {}: {}'.format(host, e))
            log.error('SSH settings: {}'.format(dict(ssh_settings)))
            sys.exit(-1)

    def connect(self):
        if self.ssh_tunnel:
            db_settings = dict(self.db_settings)
            db_settings['host'] = 'localhost'
            db_settings['port'] = self.ssh_tunnel.local_bind_port
            return self._connect(**db_settings)
        else:
            return self._connect(**self.db_settings)

    def _connect(self, dbname, dbuser, host, password, port, ssh_gateway=None):
        cons = "dbname='{dbname}' user='{dbuser}' host='{host}' " \
               "password='{password}' port='{port}'".format(**locals())
        if ssh_gateway:
            log.info('Connecting to db {dbname} at {host}:{port} through gateway {gateway}'.format(
                host=self.db_settings.get('host'),
                port=self.db_settings.get('port'),
                dbname=dbname,
                gateway=ssh_gateway
                )
            )
        else:
            log.info('Connecting to db {dbname} at {host}:{port}'.format(**locals()))
        try:
            return psycopg2.connect(cons)
        except psycopg2.OperationalError as e:
            log.error('Error connecting to database: {}'.format(e))
            sys.exit(-1)

    @property
    def output_folder(self):
        return os.path.join(
            self.cfg['Reports']['destination_folder'],
            self.name)

    def exec_query(self, query):
        self.cur.execute(query)
        return self.cur.fetchall()

    def generate_report(self):
        self.cur.execute(self.query)
        results = list()
        for row in self.cur.fetchall():
            results.append(dict(zip(self.table_columns, row)))
        return results

    @property
    def report(self):
        if not self._report:
            self._report = self.generate_report()
        return self._report

    def output_file(self, ext='json'):
        return os.path.join(
            self.output_folder,
            '{}.{}'.format(self.name, ext)
        )

    def save_to_csv(self, output_file=None):
        csv_query = "COPY ({0}) TO STDOUT WITH CSV HEADER".format(self.query)
        csv_file = output_file or self.output_file('csv')
        log.info('Saving query results to CSV file: {}'.format(csv_file))
        mkpath(os.path.dirname(csv_file))
        with open(csv_file, 'w', encoding='ascii') as f:
            self.cur.copy_expert(csv_query, f)

    def save_to_json(self):
        out_json = self.output_file()
        log.info('Saving to JSON: {}'.format(out_json))
        mkpath(self.output_folder)
        with open(out_json, 'w', encoding='utf8') as outfile:
            json.dump(self.report, outfile)

    def save_to_html(self, template_name=None, **args):
        out_html = self.output_file(ext='html')
        log.info('Saving to HTML: {}'.format(out_html))
        template_name = template_name or self.name + '.html'
        jinja_env = jinja2.Environment(loader=jinja2.PackageLoader('damster', 'templates'))
        template = jinja_env.get_template(template_name)

        log.debug('Using template {}'.format(template))

        html = template.render(report=self.report, **args)
        mkpath(self.output_folder)
        with open(out_html, 'w', encoding='utf8') as outfile:
            outfile.write(html)

    def __del__(self):
        if 'cur' in self.__dict__:
            self.cur.close()
            self.db.close()
        if self.ssh_tunnel:
            self.ssh_tunnel.stop()



from damster.utils import initialize_logger
import re
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

log = initialize_logger(__name__)


def _sanitize_url(url):
    u = urlparse(url)
    return u.netloc.split(':')[0].replace('.', '_')


class BambooBuildAgentsMetrics(object):

    AGENTS_BUILDING = re.compile(r'(\d+) of (\d+) online agents building')

    def __init__(self, bamboo_client, influx_client):
        self.cli = bamboo_client
        self.influx = influx_client

    def agent_status(self, metric_name):
        try:
            agents = self.cli.agent_status()
            log.debug('Agent Status: {}'.format(agents))
            metrics = list()
            for agent in agents:
                metrics.append(dict(
                    measurement=metric_name,
                    tags=dict(
                        agent_name=agent['name'].replace(' ', '_'),
                        type=agent['type'],
                        busy=agent['busy'],
                        enabled=agent['enabled'],
                        active=agent['active'],
                    ),
                    fields=dict(
                        value=1
                    )
                ))
            self.influx.write_points(metrics)
        except ValueError as e:
            log.error('Agent Status threw a ValueError: {}'.format(e))
        except Exception as e:
            log.error('Agent Status threw an Exception: {}'.format(e))

    def activity(self, metric_name, tags=None):
        try:
            activity = self.cli.activity()
            log.debug('Build Activity: {}'.format(activity))

            building = [b for b in activity['builds'] if b['status'] == 'BUILDING']
            queued = [b for b in activity['builds'] if b['status'] == 'QUEUED']
            total_building = len(building)
            local_building = len([b for b in building if b['agent']['type'] == 'LOCAL'])
            remote_building = len([b for b in building if b['agent']['type'] == 'REMOTE'])
            total_queued = len(queued)
            try:
                search = re.search(self.AGENTS_BUILDING, activity['agentSummary'])
                agents_building = int(search.group(1))
                agents_online = int(search.group(2))
            except AttributeError:
                log.error('Error parsing agentSummary')
                agents_building = 0
                agents_online = 0

            metric = dict(
                    measurement=metric_name,
                    tags=dict(
                        host=_sanitize_url(self.cli.url)
                    ),
                    fields=dict(
                        total_building=total_building,
                        local_building=local_building,
                        remote_building=remote_building,
                        queued=total_queued,
                        agents_building=agents_building,
                        agents_online=agents_online
                    )
                )
            if tags:
                metric['tags'].update(tags)
            log.debug('Metric: {}'.format(metric))
            self.influx.write_points([metric])
        except Exception as e:
            log.error('Bamboo Activity threw an Excepton: {}'.format(e))



import logging
import os
import sys
import pkg_resources
import arrow
from arrow.parser import ParserError
from configparser import ConfigParser, ExtendedInterpolation
import re

NON_ASCII = re.compile(r'[^\x00-\x7f]')


class LogFilter(logging.Filter):
    """
    Used by :func:`initialize_logger` to redirect errors to ``stderr``
    """
    def filter(self, rec):
        return rec.levelno in (logging.DEBUG, logging.INFO, logging.WARNING)


def initialize_logger(module_name):
    """
    Log initialization

    :param module_name: module name
    :return: logger object
    """

    levels = {'DEBUG': logging.DEBUG,
              'INFO': logging.INFO,
              'WARNING': logging.WARNING,
              'ERROR': logging.ERROR}
    log_level = levels[os.environ.get('DAMSTER_LOGLEVEL', 'INFO')]
    logger = logging.getLogger(module_name)

    log_fmt_string = '%(asctime)-19s %(name)s: [%(levelname)-7s] - %(message)s'
    formatter = logging.Formatter(log_fmt_string)

    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setLevel(logging.DEBUG)
    stdout_handler.addFilter(LogFilter())
    stdout_handler.setFormatter(formatter)
    stderr_handler = logging.StreamHandler(sys.stderr)
    stderr_handler.setLevel(logging.ERROR)
    stderr_handler.setFormatter(formatter)
    logger.propagate = 0
    logger.addHandler(stdout_handler)
    logger.addHandler(stderr_handler)
    logger.setLevel(log_level)
    return logger


log = initialize_logger(__name__)


def get_config(config_file):
    config = ConfigParser(interpolation=ExtendedInterpolation())
    defaults = pkg_resources.resource_filename('damster', 'defaults.cfg')
    cwd = os.getcwd()
    config_files_list = [defaults, os.path.expanduser('~/.config/damster.cfg'), os.path.join(cwd, 'damster.cfg')]
    if config_file:
        config_file_abs = os.path.join(cwd, config_file)
        if os.path.isfile(config_file_abs):
            config_files_list.append(config_file_abs)
        else:
            log.error('Can not read config file "{}"'.format(config_file_abs))
            sys.exit(-1)
    config.read(config_files_list)
    return config


# TODO: Make this use global time zone
def time_to_excel(tm, tzinfo='Europe/Amsterdam'):
    try:
        if type(tm) != arrow.arrow.Arrow:
            tm = arrow.get(tm).replace(tzinfo=tzinfo)
        temp = arrow.get('1899-12-30').replace(tzinfo=tzinfo)  # Note, not 31st Dec but 30th!
        delta = arrow.get(tm) - temp
    except ParserError:
        return ''
    return str(float(delta.days) + (float(delta.seconds) / 86400))


def time_delta(t1, t2, excel=True):
    try:
        at1 = arrow.get(t1)
        at2 = arrow.get(t2)
        delta = at2 - at1
        if excel:
            delta = str(float(delta.days) + (float(delta.seconds) / 86400))
        return delta
    except ParserError:
        if excel:
            return '0'
        return 0


def quoted(it):
    return '"{}"'.format(it)


def replace_non_ascii(s, replacement=''):
    return re.sub(NON_ASCII, replacement, s)



#!/bin/env python
from damster.utils import initialize_logger, get_config
from damster.metrics.bamboo import BambooBuildAgentsMetrics
from atlassian import Bamboo
from influxdb import InfluxDBClient

import schedule
import time

log = initialize_logger(__name__)


cfg = get_config()


def main():
    log.setLevel(int(cfg['Common']['loglevel']))

    bamboo = Bamboo(**cfg['Bamboo'])
    influxdb = InfluxDBClient(**cfg['InfluxDB'])

    bamboo_metrics = BambooBuildAgentsMetrics(bamboo, influxdb)

    log.debug('Scheduling jobs...')
    schedule.every(1).minutes.do(
        bamboo_metrics.agent_status, metric_name='wst_swf_bamboo_agent_status')
    schedule.every(1).minutes.do(
        bamboo_metrics.activity, metric_name='wst_swf_bamboo_activity')

    while True:
        schedule.run_pending()
        time.sleep(1)


def deployment_report():
    bamboo = Bamboo(**cfg['Bamboo'])
    report = BambooDeploymentsReport(bamboo, '2018-01-18 00:00:00')
    report.run_report()


if __name__ == "__main__":
    main()



from damster.utils import initialize_logger, time_to_excel, time_delta, quoted, replace_non_ascii
from damster.reports.bamboo.utils import TriggerReason
from damster.reports.base_report import BaseReport
from atlassian import Bamboo
import jinja2
from distutils.dir_util import mkpath
import os
import json

log = initialize_logger(__name__)


class BambooDeploymentsReport(BaseReport):

    def __init__(self, cfg, from_date, to_date=None, name='bamboo_deployments_report'):
        super(BambooDeploymentsReport, self).__init__(cfg, from_date, to_date, name)
        self.bamboo = Bamboo(**cfg['Bamboo'])

    # TODO: Needs refactoring
    def generate_report(self):
        log.info('Starting report generation for deployment results between {} and {}'.format(
            self._time_to_string(self.from_date), self._time_to_string(self.to_date)
        ))
        report = list()
        for project in self.bamboo.deployment_projects():
            log.debug('Searching deployments for project "{}"'.format(project['name']))
            plan_key = project['planKey']['key'] if 'planKey' in project else ''
            project_dict = dict(
                prj_id=project['id'],
                prj_name=project['name'],
                prj_plan=plan_key,
                prj_environments=list()
            )
            log.debug(project_dict)
            for env in project['environments']:
                env_dict = dict(
                    env_id=env['id'],
                    env_name=env['name'],
                    env_results=list()
                )
                log.debug('Getting deployments to environment "{}"'.format(env['name']))
                for result in self.bamboo.deployment_environment_results(env['id'], expand='results', max_results=25):
                    started_time = self._long_time_to_epoch(result['startedDate'])
                    if started_time > self.from_date:
                        if started_time < self.to_date:
                            env_dict['env_results'].append(self.get_deploy_result_details(result))
                    else:
                        break  # Since results are ordered from newest, exit as soon as we find an older result
                env_summary = dict(
                    successful=len([r for r in env_dict['env_results'] if r['state'] == 'SUCCESS']),
                    failed=len([r for r in env_dict['env_results'] if r['state'] == 'FAILED']),
                    in_progress=len([r for r in env_dict['env_results'] if r['state'] == 'UNKNOWN']),
                )
                env_dict['summary'] = env_summary
                project_dict['prj_environments'].append(env_dict)
            project_dict['summary'] = dict(
                successful=sum([e['summary']['successful'] for e in project_dict['prj_environments']]),
                failed=sum([e['summary']['failed'] for e in project_dict['prj_environments']]),
                in_progress=sum([e['summary']['in_progress'] for e in project_dict['prj_environments']]),
            )
            report.append(project_dict)
        return report

    def get_deploy_result_details(self, result):
        finished_time = self._long_time_to_epoch(result['finishedDate']) \
            if 'finishedDate' in result else ''
        queued_time = self._long_time_to_epoch(result['queuedDate']) \
            if 'queuedDate' in result else ''
        executed_time = self._long_time_to_epoch(result['executedDate']) \
            if 'executedDate' in result else ''
        trigger, user_name, user_id, build_id = TriggerReason(result['reasonSummary']).tuple
        result_dict = dict(
            id=result['id'],
            started=self._time_to_string(self._long_time_to_epoch(result['startedDate'])),
            finished=self._time_to_string(finished_time),
            queued=self._time_to_string(queued_time),
            executed=self._time_to_string(executed_time),
            state=result['deploymentState'],
            version_name=result['deploymentVersionName'],
            deployment_type=trigger,
            deployment_type_raw=result['reasonSummary'],
            deployment_trigger_user_id=user_id,
            deployment_trigger_user=user_name,
            deployment_trigger_build=build_id,
            details=self.get_build_details_from_result(result)
        )
        log.debug('Processing result {}'.format(result_dict))
        return result_dict

    def get_build_details_from_result(self, result):
        details = list()
        if not('deploymentVersion' in result and 'items' in result['deploymentVersion']):
            log.error('No version details available for {}'.format(result))
            return details
        try:
            for artifact in result['deploymentVersion']['items']:
                log.debug('Getting info for plan: {}'.format(artifact))
                build_key = artifact['planResultKey']['key']
                if not build_key.startswith('DELETED_'):
                    build = self.bamboo.build_result(build_key)
                    trigger, user_name, user_id, trigger_plan_key = TriggerReason(build['buildReason']).tuple
                    build_reason = build['buildReason']
                else:
                    trigger, user_name, user_id, trigger_plan_key = '', '', '', build_key
                    build_reason = 'N/A'
                details.append(
                    dict(
                        plan_key=build_key,
                        trigger=trigger,
                        user_id=user_id,
                        user_name=user_name,
                        trigger_plan_key=trigger_plan_key,
                        build_reason=build_reason
                    )
                )
        except Exception as e:
            log.error(e)
        return details

    def save_to_csv(self):
        out_csv = self.output_file(ext='csv')
        log.info('Saving to CSV file {}'.format(out_csv))
        lines = list()
        header = ','.join([
            'prj_id',
            'prj_name',
            'prj_plan_key',
            'env_name',
            'status',
            'started',
            'finished',
            'queued',
            'executed',
            'total_time',
            'time_in_queue',
            'time_deploying',
            'version_name',
            'deployment_type',
            'deployment_trigger_user_name',
            'deployment_trigger_user_id',
            'deployment_trigger_build',
            'deployment_type_raw'
        ])
        lines.append(header)
        for project in self.report:
            for environment in project['prj_environments']:
                for result in environment['env_results']:
                    try:
                        line = ','.join([
                            str(project['prj_id']),
                            quoted(project['prj_name']),
                            project['prj_plan'],
                            quoted(environment['env_name']),
                            result['state'],
                            time_to_excel(result['started']),
                            time_to_excel(result['finished']),
                            time_to_excel(result['queued']),
                            time_to_excel(result['executed']),
                            time_delta(result['started'], result['finished']),
                            time_delta(result['queued'], result['executed']),
                            time_delta(result['executed'], result['finished']),
                            result['version_name'],
                            result['deployment_type'],
                            quoted(result['deployment_trigger_user']),
                            quoted(result['deployment_trigger_user_id']),
                            result['deployment_trigger_build'],
                            result['deployment_type_raw'].replace(', ', '; ').replace('\n', ' - ')
                        ])
                        lines.append(line)
                    except TypeError as e:
                        log.error(e)
                        log.error(result)

        mkpath(self.output_folder)
        with open(out_csv, 'w', encoding='ascii') as outfile:
            outfile.write(replace_non_ascii('\n'.join(lines), '_'))

    def filter_projects_with_no_deployments(self):
        return [prj for prj in self.report if sum(
            [prj['summary']['successful'], prj['summary']['failed'], prj['summary']['in_progress']]) > 0]

    def save_to_html(self, template_name=None, filter_empty=True, **args):
        out_html = self.output_file(ext='html')
        log.info('Saving to HTML: {}'.format(out_html))
        template_name = template_name or self.name + '.html'
        jinja_env = jinja2.Environment(loader=jinja2.PackageLoader('damster', 'templates'))
        template = jinja_env.get_template(template_name)
        report = self.filter_projects_with_no_deployments() if filter_empty else self.report

        summary = dict(
            projects=len(report),
            successful=sum([pr['summary']['successful'] for pr in report]),
            failed=sum([pr['summary']['failed'] for pr in report]),
            in_progress=sum([pr['summary']['in_progress'] for pr in report]),
            from_date=self._time_to_string(self.from_date, fmt='YYYY/MM/DD'),
            to_date=self._time_to_string(self.to_date, fmt='YYYY/MM/DD'),
            bamboo_url=self.bamboo.url
        )

        html = template.render(
            deployments=report,
            summary=summary,
            **args
        )
        mkpath(self.output_folder)
        with open(out_html, 'w', encoding='utf8') as outfile:
            outfile.write(html)

    def run_report(self, use_cache=True):
        if not (os.path.isfile(self.output_file()) and use_cache):
            self._report = self.generate_report()
            self.save_to_json()
        else:
            self._report = json.load(open(self.output_file()))

        self.save_to_csv()

        self.save_to_html(title='Bamboo Reports: Deployments')



from damster.utils import initialize_logger
from damster.reports.db_query import GenericDB
from atlassian import Crowd


from distutils.dir_util import mkpath

log = initialize_logger(__name__)


class BambooDBDeploymentPermissions(GenericDB):

    query_project = """
              SELECT
                DP.name as deployment_project,
                DP.deployment_project_id as deployment_project_id,
                AE.sid  as entity_name,
                (CASE
                  WHEN AE.type = 'PRINCIPAL'
                    THEN 'USER'
                  WHEN AE.type = 'GROUP_PRINCIPAL'
                    THEN 'GROUP'
                  ELSE
                     'ROLE'
                END) as entity_type,
                sum(mask) as mask
              FROM ACL_ENTRY AS AE
                JOIN ACL_OBJECT_IDENTITY AS AOI
                  ON AE.acl_object_identity = AOI.id
                JOIN DEPLOYMENT_PROJECT DP ON deployment_project_id = object_id_identity
              GROUP BY
                deployment_project, deployment_project_id, entity_name, entity_type
              ORDER BY
                deployment_project
    """
    query_environment = """
            SELECT
                DP.name as deployment_project,
                DP.deployment_project_id as deployment_project_id,
                DE.name as deployment_environment,
                AE.sid as entity_name,
                (CASE
                    WHEN AE.type = 'PRINCIPAL'
                       THEN 'USER'
                    WHEN AE.type = 'GROUP_PRINCIPAL'
                       THEN 'GROUP'
                    ELSE
                       'ROLE'
                END) as entity_type,
                sum(mask) as mask
            FROM ACL_ENTRY AS AE
            JOIN ACL_OBJECT_IDENTITY AS AOI
                ON AE.acl_object_identity = AOI.id
            JOIN DEPLOYMENT_ENVIRONMENT DE
                ON DE.environment_id = object_id_identity
            JOIN DEPLOYMENT_PROJECT DP
                ON DE.package_definition_id = DP. deployment_project_id
            GROUP BY
                deployment_project, deployment_project_id, deployment_environment, entity_type, entity_name
            ORDER BY
                deployment_project
            """
    name = 'deployment_permissions'

    def __init__(self, cfg, db_settings_section='Bamboo DB',
                 name='deployment_permissions',
                 use_ssh_tunnel=False):
        super(BambooDBDeploymentPermissions, self).__init__(cfg, db_settings_section, name,
                                                            use_ssh_tunnel=use_ssh_tunnel)
        self.crowd = Crowd(**cfg['Crowd'])

    @staticmethod
    def _mask_to_dict(mask):
        permissions = [
            ('view', 1),
            ('edit', 2),
            ('deploy', 64),
            # No clone or admin for deployments(
            # ('admin', 16),
            # ('clone', 128)
        ]
        perms = dict()
        for perm, int_mask in permissions:
            perms[perm] = int_mask & int(mask) > 0
        return perms

    def _get_display_name(self, user_id):
        try:
            user_details = self.crowd.user(user_id)
            return user_details['display-name']
        except Exception:
            return user_id

    def generate_report(self):
        report = dict()

        # First get the project permissions
        dep_projects_perms = self.exec_query(query=self.query_project)
        for deployment_project, id, entity_name,  entity_type, mask in dep_projects_perms:
            if deployment_project not in report:
                report[deployment_project] = dict(
                    id=id,
                    permissions=list(),
                    environments=dict()
                )
            permission = self._mask_to_dict(mask)
            permission['entity_name'] = entity_name
            permission['entity_type'] = entity_type
            permission['display_name'] = self._get_display_name(entity_name) if entity_type == 'USER' else entity_name
            report[deployment_project]['permissions'].append(permission)

        # Second, get the environment permissions
        dep_environments_perms = self.exec_query(self.query_environment)
        for deployment_project, id, deployment_environment, entity_name, entity_type, mask in dep_environments_perms:
            if deployment_project not in report:
                report[deployment_project] = dict(
                    id=id,
                    prj_permissions=list(),
                    environments=dict()
                )
            if deployment_environment not in report[deployment_project]['environments']:
                report[deployment_project]['environments'][deployment_environment] = list()
            permission = self._mask_to_dict(mask)
            permission['entity_name'] = entity_name
            permission['entity_type'] = entity_type
            permission['display_name'] = self._get_display_name(entity_name) if entity_type == 'USER' else entity_name
            report[deployment_project]['environments'][deployment_environment].append(permission)

        return report

    def save_to_csv(self, output_file=None):
        out_csv = output_file or self.output_file(ext='csv')
        log.info('Saving to CSV file {}'.format(out_csv))
        lines = ['deployment_project,environment_name,entity_name,entity_type,view,edit,deploy']
        for deployment_project in self.report:
            line = '{},{},_PROJECT_,'.format(self.report[deployment_project]['id'], deployment_project)
            for project_permission in self.report[deployment_project]['permissions']:
                lines.append(
                    line + '"{entity_name}","{display_name}",{entity_type},{view},{edit},{deploy}'.format(
                        **project_permission))
            for environment in self.report[deployment_project]['environments']:
                line = '{},{},{},'.format(self.report[deployment_project]['id'], deployment_project, environment)
                for env_permission in self.report[deployment_project]['environments'][environment]:
                    lines.append(
                        line + '"{entity_name}","{display_name}",{entity_type},{view},{edit},{deploy}'.format(
                            **env_permission))

        mkpath(self.output_folder)
        with open(out_csv, 'w', encoding='utf8') as outfile:
            outfile.write('\n'.join(lines))



from damster.utils import initialize_logger
import arrow
from distutils.dir_util import mkpath
import os
import json

log = initialize_logger(__name__)


class BaseReport(object):

    def __init__(self, cfg, from_date, to_date=None, name='base_report'):

        self.name = name
        self.cfg = cfg
        self.time_zone = cfg['Common']['time_zone']
        self.from_date = arrow.get(from_date).replace(tzinfo=self.time_zone)
        self.to_date = arrow.get(to_date).replace(tzinfo=self.time_zone) \
            if to_date else arrow.utcnow().to(self.time_zone)
        self._report = None
        # self.bamboo = Bamboo(**cfg['Bamboo'])

    @property
    def output_folder(self):
        return os.path.join(
            self.cfg['Reports']['destination_folder'],
            self.name,
            '{}__{}'.format(self.from_date.format('YYYY-MM-DD'), self.to_date.format('YYYY-MM-DD'))
        )

    @property
    def report(self):
        if not self._report:
            self._report = self.generate_report()
        return self._report

    def output_file(self, ext='json'):
        return os.path.join(
            self.output_folder,
            '{}.{}'.format(self.name, ext)
        )

    def save_to_json(self):
        out_json = self.output_file()
        log.info('Saving to JSON: {}'.format(out_json))
        mkpath(self.output_folder)
        with open(out_json, 'w', encoding='utf8') as outfile:
            json.dump(self.report, outfile)

    def generate_report(self):
        return self._report  # This will be overridden

    def run_report(self, use_cache=True):
        if not (os.path.isfile(self.output_file()) and use_cache):
            self.generate_report()
            self.save_to_json()
        else:
            self._report = json.load(open(self.output_file()))

    def _long_time_to_epoch(self, tm):
        """
        Converts long epoch (from Bamboo deployments timestamps) into 'normal' epoch
        and sets to current time zone
        :param tm: long
        :return: arrow time object
        """
        return arrow.get(tm / 1000).to(self.time_zone)

    def _time_to_string(self, tm, fmt='YYYY-MM-DD HH:mm:ss'):
        """
        Converts arrow time to string
        :param tm: arrow time object
        :param fmt: format string
        :return: string with formatted time
        """
        if tm:
            return tm.to(self.time_zone).format(fmt)
        return tm



#!/bin/env python
from damster.utils import initialize_logger, get_config
from damster.reports.bamboo import BambooDeploymentsReport

log = initialize_logger(__name__)


cfg = get_config()

YEAR = '2017'


def monthly():

    for month in range(1, 11):
        report = BambooDeploymentsReport(
            cfg,
            from_date='{}-{:02}-01'.format(YEAR, month),
            to_date='{}-{:02}-01'.format(YEAR, month+1)
        )
        report.run_report(use_cache=True)


def main():

    report = BambooDeploymentsReport(
        cfg,
        from_date='2018-01-01',
        # to_date='2018-02-01'
    )
    report.run_report(use_cache=False)


if __name__ == "__main__":
    main()



import os
from setuptools import find_packages, setup

version_file = os.path.join('damster', 'VERSION')


setup(
    name='damster',
    description='Reports and metrics for Atlassian Tools',
    long_description=open('README.md', 'r').read(),
    license='Apache License 2.0',
    version=open(version_file, 'r').read().strip(),
    download_url='https://github.com/cattz/damster',

    author='Xabier Davila',
    author_email='davila.xabier@gmail.com',

    packages=find_packages(),
    package_dir={'damster': 'damster'},
    include_package_data=True,

    zip_safe=False,

    install_requires=[
        'configparser',
        'arrow',
        'click',
        'atlassian-python-api',
        'arrow',
        'influxdb',
        'schedule',
        'jinja2',
        'psycopg2',
        'sshtunnel',
        'six'
    ],

    extras_require={
        'dev': [
            'pytest',
            'pytest-pep8',
            'coverage',
            'tox'
        ],
    },
    platforms='Platform Independent',

    entry_points={
        "console_scripts": [
            "damster    = damster.cli:cli",
        ]
    },

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
    ]
)



from damster.reports.bamboo.utils import TriggerReason


class TestParseTriggerReason(object):

    reasons = [
        ('Child of <a href="https://host/path/to/XX-YY-55">XX-YY-55</a>', ('Child', '', '', 'XX-YY-55')),
        ('Manual run by <a href="https://host/path/to/userid">User Name</a>', ('Manual', 'User Name', 'userid', '')),
        ('Manual run by someuser', ('Manual', 'someuser', 'someuser', '')),
        ('Code has changed', ('Commit', '', '', '')),
        ('Scheduled', ('Scheduled', '', '', '')),
        ('Changes by <a href="http://url/viewUserSummary.action?currentUserName=usid">Last, First</a>',
         ('Commit', 'Last, First', 'usid', '')),
        ('Rebuilt by <a href="https://foo.bar.com/bamboo/browse/user/jsnow">John Snow</a>',
         ('Rebuilt', 'John Snow', 'jsnow', '')),
        ('Rebuilt      by <a href="https://foo.bar.com/bamboo/browse/user/mward">M Ward</a>',
         ('Rebuilt', 'M Ward', 'mward', '')),
        ('Code changes detected', ('Commit', '', '', '')),
        ('First build for this plan', ('First build', '', '', '')),
        ('Triggered by <a href="https://foobar.com/bamboo/deploy/viewDeploymentResult.action?'
         'deploymentResultId=1122334455">Some Deployment Project &rsaquo; Test</a>',
         ('Child', '', '', 'Some Deployment Project &rsaquo; Test')),
        ('Custom build by SOUSERID&nbsp;with revision '
         '<a href="https://foobar.com/bamboo/browse/PPP-LLL-777#changesSummary">1234abc</a>',
         ('Manual', 'SOUSERID', 'SOUSERID', '')),
        ('Manual run from the stage: <b>Deploy</b> by <a href="https://swfactory.aegon.com/bamboo/browse/user/UUID">'
         'User Name</a>', ('Manual', 'User Name', 'UUID', '')),
        ('Manual run from the stage: <b>Artifactory</b> by L4IJUM', ('Manual', 'L4IJUM', 'L4IJUM', ''))
    ]

    def test_dummy(self):
        for reason, result in self.reasons:
            assert result == TriggerReason(reason).tuple



from damster.utils import initialize_logger
try:
    from urllib import unquote  # Python 2.X
except ImportError:
    from urllib.parse import unquote  # Python 3+
import re

log = initialize_logger(__name__)


class TriggerReason(object):
    """
    Parses Bamboo trigger reason strings
    Examples of possible buildReason string:
    Child of <a href="https://.../FHS-FHSADP-55">FHS-FHSADP-55</a>
    Manual run by <a href="https://...">Davila Freire, Xabier</a>
    Manual run by someuser
    Code has changed
    Scheduled
    """

    manual = re.compile(
        r'Manual run (?:.+?(?=by))?by\s*(?:<a href="http[s]?:\/\/[^"]*\/(?P<user_id>.*)">)?'
        r'(?P<user_name>[^<]*)(?:<\/a>)?$')
    child = re.compile(
        r'(?:Child\s+of\s+|Triggered\s+by\s+)<a href="http[s]?:\/\/[^"]*">(?P<parent_plan>.*)<\/a>')
    source = re.compile(
        r'Changes\s+by\s+(?:<a href="http[s]?:\/\/[^=]*=(?P<user_id>.*)">)?(?P<user_name>[^<]*)(?:<\/a>)?'
    )
    rebuilt = re.compile(
        r'Rebuilt\s+by\s+(?:<a href="http[s]?:\/\/[^"]*\/(?P<user_id>.*)">)?(?P<user_name>[^<]*)(?:<\/a>)?'
    )
    custom_build = re.compile(
        r'^Custom build by (?P<user_id>[^&]+).*$'
    )

    def __init__(self, msg):

        self.msg = unquote(msg)
        self.trigger_type, self.user_id, self.user_name, self.build_key = self.parse()

    def parse(self):
        if self.msg.startswith('Code has changed') or self.msg.startswith('Code changes detected'):
            return 'Commit', '', '', ''

        if self.msg.startswith('Scheduled'):
            return 'Scheduled', '', '', ''

        if self.msg.startswith('First build for this plan'):
            return 'First build', '', '', ''

        found = re.search(self.manual, self.msg)
        if found:
            user_id = found.group('user_id') or found.group('user_name')
            return 'Manual', user_id, found.group('user_name'), ''

        found = re.search(self.child, self.msg)
        if found:
            return 'Child', '', '', found.group('parent_plan')

        found = re.search(self.source, self.msg)
        if found:
            user_id = found.group('user_id') or found.group('user_name')
            return 'Commit', user_id,  found.group('user_name'), ''

        found = re.search(self.rebuilt, self.msg)
        if found:
            user_id = found.group('user_id') or found.group('user_name')
            return 'Rebuilt', user_id,  found.group('user_name'), ''

        found = re.search(self.custom_build, self.msg)
        if found:
            user_id = found.group('user_id')
            return 'Manual', user_id, user_id, ''

        else:
            log.error('No regex matching: {}'.format(self.msg))
            # raise ValueError('No regex matching: {}'.format(self.msg))
            return '', '', '', ''

    @property
    def tuple(self):
        return self.trigger_type, self.user_name, self.user_id, self.build_key

    @property
    def dict(self):
        return dict(
            type=self.trigger_type,
            user_name=self.user_name,
            user_id=self.user_id,
            build=self.build_key
        )
